"""
CNBV Catalogs - Sectores y entidades reguladas por la CNBV.

Comisión Nacional Bancaria y de Valores
"""

from .sectores import ReguladorFinanciero, SectoresCNBV

__all__ = ["SectoresCNBV", "ReguladorFinanciero"]
